// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const GM_CC = Components.classes;
const GM_CI = Components.interfaces;

function gmParser() {}
gmParser.prototype = {
  _converter: null,
  _domParser: null,
  _domSerializer: null,
  _doc: null,
  
  get doc() {
    if (!this._doc)
      this.load();
    return this._doc;
  },
  
  load: function()
  {
    // Load services
    this._converter = GM_CC["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(GM_CI.nsIScriptableUnicodeConverter);
    this._domParser = GM_CC['@mozilla.org/xmlextras/domparser;1'].getService(GM_CI.nsIDOMParser);
    this._domSerializer = GM_CC['@mozilla.org/xmlextras/xmlserializer;1'].getService(GM_CI.nsIDOMSerializer);
    
    // Initialize converter
    this._converter.charset = "UTF-8";
    
    // Load default
    this.open(null);
  },
  
  open: function(aFile)
  {
    var file = (aFile != null ? aFile : this._getPrefsFile(false));
    var fiStream = null;
    var siStream = null;
    var data = new String();
    
    // Get back up file
    if (!file.exists())
      file = this._getPrefsFile(true);
    
    // Create new file
    if (file.exists())
    {
      try {
        fiStream = GM_CC["@mozilla.org/network/file-input-stream;1"].createInstance(GM_CI.nsIFileInputStream);
        siStream = GM_CC["@mozilla.org/scriptableinputstream;1"].createInstance(GM_CI.nsIScriptableInputStream);
        
        fiStream.init(file, 1, 0, false);
        siStream.init(fiStream);
        
        while (siStream.available() > 0)
        {
          var chunk = siStream.read(siStream.available());
          data += this._converter.ConvertToUnicode(chunk);      
        }
      }
      catch(e) {
        // Ensure no garbage data
        data = null;
      }
      finally {
        if (fiStream != null)
          fiStream.close();
        if (siStream != null)
          siStream.close();
      }
    }
    else
      data = this._defaultPrefsFile();
    
    if (data != null)
    {
      var tempDoc = this._domParser.parseFromString(data, "text/xml");
      
      // Validates the prefs file
      if (this._validatePrefsFile(tempDoc))
        this._doc = tempDoc;
      else
        data = null;
      
      if (!this._getPrefsFile(false).exists())
        this.save(null);
    }
    
    return (data != null);
  },
  
  save: function(aFile)
  {
    var file = (aFile != null ? aFile : this._getPrefsFile(false));
    var content = this._domSerializer.serializeToString(this._doc);
    var foStream = null;
    
    if (aFile == null && this._getPrefsFile(false).exists())
      this._getPrefsFile(false).moveTo(null, "prefs.bak");
    
    try {
      // Creates the file out stream instance
      foStream = GM_CC["@mozilla.org/network/file-output-stream;1"].createInstance(GM_CI.nsIFileOutputStream);
      
      // Initializes the file out stream to write
      foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
      
      // Write content to file
      var chunk = this._converter.ConvertFromUnicode(content);
      foStream.write(chunk, chunk.length);
      
      // Check if any content is left
      var fin = this._converter.Finish();
      if (fin.length > 0)
        foStream.write(fin, fin.length);
    }
    catch(e) {}
    finally {
      if (foStream != null)
        foStream.close();
    }
  },
  
  getAccountNode: function()
  {
    var content = '  <account type="" email="" password="" alias="">\n' +
                  '    <pref id="general-automatic-login" type="Bool" value="false"/>\n' +
                  '    <pref id="general-secured-connection" type="Bool" value="true"/>\n' +
                  '    <pref id="toolbar-account-hide-unread-count" type="Bool" value="false"/>\n' +
                  '    <pref id="toolbar-account-hide-alias" type="Bool" value="false"/>\n' +
                  '    <pref id="toolbar-tooltip-show-labels" type="Bool" value="true"/>\n' +
                  '    <pref id="toolbar-tooltip-show-snippets" type="Bool" value="true"/>\n' +
                  '    <pref id="toolbar-tooltip-show-accounts" type="Bool" value="false"/>\n' +
                  '    <pref id="toolbar-unread-count-inbox" type="Bool" value="true"/>\n' +
                  '    <pref id="toolbar-unread-count-spam" type="Bool" value="false"/>\n' +
                  '    <pref id="toolbar-unread-count-labels" type="Bool" value="false"/>\n' +
                  '    <pref id="notifications-check-interval" type="Int" value="15"/>\n' +
                  '    <pref id="notifications-alerts-message-count" type="Bool" value="false"/>\n' +
                  '    <pref id="notifications-alerts-snippets" type="Bool" value="false"/>\n' +
                  '    <pref id="notifications-sounds" type="Bool" value="false"/>\n' +
                  '    <pref id="notifications-sounds-file" type="Char" value=""/>\n' +
                  '  </account>\n';
    
    return this._domParser.parseFromString(content, "text/xml").documentElement;
  },
  
  getGlobalNode: function()
  {
    return '  <account type="global">\n' +
           '    <pref id="general-never-save-passwords" type="Bool" value="false"/>\n' +
           '    <pref id="compose-tab-location" type="List" value="2"/>\n' +
           '    <pref id="compose-mailto-links" type="Bool" value="false"/>\n' +
           '    <pref id="compose-mailto-default" type="List" value="0"/>\n' +
           '    <pref id="compose-context-menu" type="Bool" value="false"/>\n' +
           '    <pref id="compose-context-menu-mailto" type="Bool" value="false"/>\n' +
           '    <pref id="compose-context-menu-position" type="Int" value="0"/>\n' +
           '    <pref id="toolbar-left-click" type="List" value="4"/>\n' +
           '    <pref id="toolbar-middle-click" type="List" value="2"/>\n' +
           '    <pref id="toolbar-tab-location" type="List" value="2"/>\n' +
           '    <pref id="toolbar-reset-unread-count" type="Bool" value="false"/>\n' +
           '    <pref id="toolbar-statusbar-display" type="Bool" value="true"/>\n' +
           '    <pref id="toolbar-statusbar-always-last" type="List" value="0"/>\n' +
           '    <pref id="toolbar-statusbar-position" type="Int" value="0"/>\n' +
           '    <pref id="notifications-switch-account" type="Bool" value="false"/>\n' +
           '    <pref id="notifications-clickable-alerts" type="Bool" value="true"/>\n' +
           '  </account>\n';    
  },
  
  _defaultPrefsFile: function()
  {
    return '<?xml version="1.0"?>\n' + 
           '<gmanager>\n' +
           this.getGlobalNode() + 
           '</gmanager>';
  },
  
  _getPrefsFile: function(aBackup)
  {
    var file = GM_CC["@mozilla.org/file/directory_service;1"].getService(GM_CI.nsIProperties).get("ProfD", GM_CI.nsIFile);
    
    // Get directory
    file.append("gmanager");
    
    if (!file.exists())
      file.create(file.DIRECTORY_TYPE, 0755);
    
    // Check file
    if (!aBackup)
      file.append("prefs.xml");
    else
      file.append("prefs.bak");
    
    return file;
  },
  
  _validatePrefsFile: function(aDoc)
  {
    // Loop through the opened XML file and make sure
    // the account type "global" exists and that the id
    // attributes match up with the defaultGlobal attributes
    
    var valid = false;
    var chkAccounts = aDoc.getElementsByTagName("account");
    
    // Check for account tag
    if (chkAccounts.length > 0)
    {
      var chkGlobal = null;
      
      for (var i = 0; i < chkAccounts.length && chkGlobal == null; i++)
        if (chkAccounts[i].getAttribute("type") == "global")
          chkGlobal = chkAccounts[i];
      
      if (chkGlobal != null)
      {
        var check = true;
        var defGlobal = this._domParser.parseFromString(this.getGlobalNode(), "text/xml");
        var defPrefs = defGlobal.getElementsByTagName("pref");
        var chkPrefs = chkGlobal.getElementsByTagName("pref");
        
        for (var i = 0; i < chkPrefs.length && check; i++)
        {
          if (defPrefs[i].getAttribute("id") != chkPrefs[i].getAttribute("id") ||
              defPrefs[i].getAttribute("type") != chkPrefs[i].getAttribute("type"))
            check = false;
        }
        
        valid = check;
      }
    }
    
    return valid;
  },
  
  QueryInterface: function(iid)
  {
    if (!iid.equals(GM_CI.gmIParser) &&
        !iid.equals(GM_CI.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{d0fe9af0-f7bc-11da-974d-0800200c9a66}"),
  myDesc: "Service for parser",
  myProgID: "@longfocus.com/gmanager/parser;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new gmParser()).QueryInterface(iid);
    }
  },
  
  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }