// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

function onOverlayLoad() {
  // Load overlay
  GM_gOverlay.load();
}

function onOverlayUnload() {
  // Unload overlay
  GM_gOverlay.unload();
}

var GM_gOverlay = {
  _prefWatch: null,
  _observer: null,
  _accounts: null,
  _current: null,
  
  load: function()
  {
    // Load manager
    GM_getManager().load();
    
    // Load preference observer
    this._prefWatch = GM_getBranch().QueryInterface(GM_CI.nsIPrefBranchInternal);
    this._prefWatch.addObserver("", this, false);
    
    // Load observers
    this._observer = GM_CC["@mozilla.org/observer-service;1"].getService(GM_CI.nsIObserverService);
    this._observer.addObserver(this, GM_NOTIFY_ACCOUNT_CHANGE, false);
    this._observer.addObserver(this, GM_NOTIFY_ACCOUNT_LOGIN, false);
    this._observer.addObserver(this, GM_NOTIFY_ACCOUNT_PREFS_CHANGE, false);
    this._observer.addObserver(this, GM_NOTIFY_ACCOUNT_RESET_UNREAD, false);
    
    // Check version
    if (GM_getBranch().getCharPref("version") != GM_getMyVersion())
      setTimeout(this._welcome, 10);
    
    // Startup
    this._startup();
  },
  
  unload: function()
  {
    var accounts = GM_getManager().getAccounts({});
    
    try {
      // Remove preference observer
      this._prefWatch.removeObserver("", this);
      
      // Remove observers
      this._observer.removeObserver(this, GM_NOTIFY_ACCOUNT_CHANGE);
      this._observer.removeObserver(this, GM_NOTIFY_ACCOUNT_LOGIN);
      this._observer.removeObserver(this, GM_NOTIFY_ACCOUNT_PREFS_CHANGE);
      this._observer.removeObserver(this, GM_NOTIFY_ACCOUNT_RESET_UNREAD);
    } catch(e) {}
    
    // Remove account observers
    for (var i = 0; i < accounts.length; i++) {
      try {
        this._observer.removeObserver(this, accounts[i].email);
      } catch(e) {}
    }
  },
  
  _welcome: function()
  {
    // Check for version 0.5 preference migration
    if (parseFloat(GM_getBranch().getCharPref("version")) < parseFloat("0.5") || GM_getBranch().getCharPref("version") == "")
      GM_migratePrefs();
    
    // Check for first time
    if (GM_getBranch().getBoolPref("first-time"))
    {
      // Open the imports window
      window.openDialog("chrome://gmanager/content/imports.xul", "imports", "centerscreen,chrome,resizable=yes");
      
      // Set first time
      GM_getBranch().setBoolPref("first-time", false);
    }
    
    // Set the current version
    GM_getBranch().setCharPref("version", GM_getMyVersion());
  },
  
  _startup: function()
  {
    // Load accounts
    this._loadAccounts();
    
    // Login accounts
    for (var email in this._accounts)
    {
      var account = this._accounts[email];
      if (account.prefs.getBoolValue("general-automatic-login"))
        account.login();
    }
    
    // Select current
    this.selectCurrent();
    
    // Check statusbar
    this._checkStatusbar();
  },
  
  _loadAccounts: function()
  {
    var accounts = GM_getManager().getAccounts({});
    
    if (this._accounts != null)
    {
      for (var email in this._accounts) {
        try {
          this._observer.removeObserver(this, email);
        } catch(e) {}
      }
    }
    
    // Create list of accounts
    this._accounts = new Array();
    
    for (var i = 0; i < accounts.length; i++)
    {
      var account = accounts[i];
      
      // Check account type
      if (account.type != "global")
      {
        // Query service
        GM_queryService(account);
        
        if (account.loggedIn)
          account.setTimer(account.prefs.getIntValue("notifications-check-interval"));
        
        try {
          this._observer.addObserver(this, account.email, false);
        } catch(e) {}
        
        // Save account
        this._accounts[account.email] = account;
      }
    }
  },
  
  select: function(aId)
  {
    switch (aId)
    {
      case "gm-overlay-menuitem-loginall":
      {
        // Login all accounts
        for (var email in this._accounts)
        {
          var account = this._accounts[email];
          
          if (!account.loggedIn)
            account.login();
        }
        break;
      }
      case "gm-overlay-menuitem-logoutall":
      {
        // Logout all accounts
        for (var email in this._accounts)
          this._accounts[email].logout();
        break;
      }
      case "gm-overlay-menuitem-all-checknow":
      {
        // Check all accounts
        for (var email in this._accounts)
        {
          var account = this._accounts[email];
          
          if (account.loggedIn)
            account.check();
        }
        break;
      }
      case "gm-overlay-menuitem-loginout":
      {
        // Login/logout account
        var account = this._accounts[this._current];
        account.loggedIn ? account.logout() : account.login();
        break;
      }
      case "gm-overlay-menuitem-checknow":
      {
        // Check/login account
        var account = this._accounts[this._current];
        account.loggedIn ? account.check() : account.login();
        break;
      }
      case "gm-overlay-menuitem-showsnippets":
      {
        // Show snippets
        GM_gAlerts.show(this._accounts[this._current]);
        break;
      }
    }
  },
  
  selectCurrent: function()
  {
    // Current account
    var current = GM_getBranch().getCharPref("current");
    this._current = (current != "" ? current : null);
    
    // Select current account
    if (this._current != null)
      this.selectAccount(this._current, false);
    else
    {
      // Toolbar
      if (GM_getEBI("gm-toolbar-button")) {
        GM_setEBIA("gm-toolbar-button", "status", GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT);
        GM_setEBIA("gm-toolbar-button", "newMail", "false");
        GM_setEBIA("gm-toolbar-button", "unread", "");
        GM_setEBIA("gm-toolbar-button", "alias", "");
      }
      
      // Statusbar
      GM_setEBIA("gm-statusbar-panel", "status", GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT);
      GM_setEBIA("gm-statusbar-panel" ,"newMail", "false");
      GM_setEBIA("gm-statusbar-panel" ,"unread", "");
      GM_setEBIA("gm-statusbar-panel" ,"alias", "");
    }
  },
  
  selectAccount: function(aEmail, aLogin)
  {
    if (this._accounts[aEmail] == null)
    {
      // Reset current
      GM_getBranch().setCharPref("current", "");
      
      // Select current
      this.selectCurrent();
      
      return;
    }
    
    var account = this._accounts[aEmail];
    var hideUnread = account.prefs.getBoolValue("toolbar-account-hide-unread-count");
    var hideAlias = account.prefs.getBoolValue("toolbar-account-hide-alias");
    var unread = this._getUnread(account.email);
    
    if (this._current != account.email)
    {
      // Current account
      this._current = account.email;
      
      // Save current account
      GM_getBranch().setCharPref("current", account.email);
    }
    
    // Login or check account
    if (aLogin)
      account.loggedIn ? account.check() : account.login();
    
    // Toolbar
    if (GM_getEBI("gm-toolbar-button")) {
      GM_setEBIA("gm-toolbar-button", "status", account.status);
      GM_setEBIA("gm-toolbar-button", "newMail", (unread > 0 ? "true" : "false"));
      GM_setEBIA("gm-toolbar-button", "unread", (!hideUnread && account.loggedIn ? unread : ""));
      GM_setEBIA("gm-toolbar-button", "alias", (!hideAlias ? GM_unicode(account.alias) : ""));
    }
    
    // Statusbar
    GM_setEBIA("gm-statusbar-panel", "status", account.status);
    GM_setEBIA("gm-statusbar-panel", "newMail", (unread > 0 ? "true" : "false"));
    GM_setEBIA("gm-statusbar-panel", "unread", (!hideUnread && account.loggedIn ? unread : ""));
    GM_setEBIA("gm-statusbar-panel", "alias", (!hideAlias ? GM_unicode(account.alias) : ""));
  },
  
  selectOption: function(aEvent, aType, aData)
  {
    var account = this._accounts[this._current];
    
    if (aEvent && (aEvent.button == 2))
      return;
    
    if (account == null) {
      window.openDialog('chrome://gmanager/content/preferences.xul', 'prefs', 'centerscreen,chrome,dependent=yes');
      return;
    }
    
    if (!account.loggedIn && aType == "account") {
      account.login();
      return;
    }
    
    switch (aType)
    {
      case "account":
      {
        var index = GM_getGlobal().getIntValue((aEvent.button == 0 ? "toolbar-left-click" : "toolbar-middle-click"));
        
        switch (index)
        {
          case 0:
            window.openDialog("chrome://gmanager/content/choices.xul", "choices", "centerscreen,chrome,dependent=yes", aType, this._current);
            break;
          case 2:
            this.select("gm-overlay-menuitem-checknow");
            break;
          case 3:
            this.selectOption(aEvent, "compose");
            break;
          case 4:
            GM_loadGmailPage("inbox", account, null, null);
            break;
        }
        
        break;
      }
      case "compose":
        var index = GM_getGlobal().getValue("compose-mailto-default");
        
        if (index == 0)
          window.openDialog("chrome://gmanager/content/choices.xul", "choices", "centerscreen,chrome,dependent=yes", aType, aData);
        else
        {
          var email = null;
          var count = 2;
          for (email in this._accounts) {
            if (count == index)
              break;
            else
              count++;
          }
          GM_loadGmailPage("compose", this._accounts[email], null, aData);
        }
        
        break;
    }
  },
  
  _getUnread: function(aEmail)
  {
    var account = this._accounts[aEmail];
    var unread = 0;
    
    if (account.prefs.getBoolValue("toolbar-unread-count-inbox"))
      unread += account.inboxUnread;
    
    if (account.prefs.getBoolValue("toolbar-unread-count-spam"))
      unread += account.spamUnread;
    
    if (account.prefs.getBoolValue("toolbar-unread-count-labels"))
    {
      var labels = account.getLabels({});
      
      for (var i = 0; i < labels.length; i++)
        unread += labels[i].unread;
    }
    
    return unread;
  },
  
  _checkStatusbar: function()
  {
    var sbar = GM_getEBI("status-bar");
    var sbarPanel = GM_getEBI("gm-statusbar-panel");
    var last = (GM_getGlobal().getIntValue("toolbar-statusbar-always-last") == 0);
    var pos = GM_getGlobal().getIntValue("toolbar-statusbar-position");
    
    sbarPanel.hidden = !GM_getGlobal().getBoolValue("toolbar-statusbar-display");
    sbar.removeChild(sbarPanel);
    
    if (last)
      sbar.appendChild(sbarPanel);
    else
      sbar.insertBefore(sbarPanel, sbar.childNodes[pos]);
  },
  
  observe: function(aSubject, aTopic, aData)
  {

//var console = GM_CC['@mozilla.org/consoleservice;1'].getService(GM_CI.nsIConsoleService);
//console.logStringMessage(aTopic);
//return;

    if (aTopic == "nsPref:changed")
    {
      switch (aData)
      {
        case "current":
        {
          // Select current
          this.selectCurrent();
          break;
        }
      }
    }
    else if (aTopic == GM_NOTIFY_ACCOUNT_CHANGE || aTopic == GM_NOTIFY_ACCOUNT_PREFS_CHANGE)
    {
      // Load accounts
      this._loadAccounts();
      
      // Select account
      this.selectAccount(this._current, false);
      
      // Check statusbar
      this._checkStatusbar();
    }
    else if (aTopic == GM_NOTIFY_ACCOUNT_LOGIN)
    {
      // Select account
      if (aData in this._accounts)
        this.selectAccount(aData, true);
      else
        this.select("gm-overlay-menuitem-loginall");
    }
    else if (aTopic == GM_NOTIFY_ACCOUNT_RESET_UNREAD)
    {
      // Toolbar
      if (GM_getEBI("gm-toolbar-button")) {
        GM_setEBIA("gm-toolbar-button", "status", GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NO_MAIL);
        GM_setEBIA("gm-toolbar-button", "newMail", "false");
        GM_setEBIA("gm-toolbar-button", "unread", "0");
      }
      
      // Statusbar
      GM_setEBIA("gm-statusbar-panel", "status", GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NO_MAIL);
      GM_setEBIA("gm-statusbar-panel", "newMail", "false");
      GM_setEBIA("gm-statusbar-panel", "unread", "0");
      
      // Reset unread count
      this._accounts[this._current].resetUnread();
    }
    else if (aTopic in this._accounts)
    {
      // Get account
      var account = this._accounts[aTopic];
      
      switch (aData)
      {
        case GM_NOTIFY_ACCOUNT_STATE_CONNECTING:
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT:
          break;
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_OUT_NO_PASSWORD:
        {
          var ww = GM_CC["@mozilla.org/embedcomp/window-watcher;1"].getService(GM_CI.nsIWindowWatcher);
          var winName = "gm-accounts-" + account.email;
          var exists = ww.getWindowByName(winName, null);
          
          if (exists == null)
            window.openDialog("chrome://gmanager/content/accounts.xul", winName, "chrome,dependent=no", "pass", account.email);
          
          break;
        }
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NEW_MAIL:
        {
          // Show snippets
          if (account.prefs.getBoolValue("notifications-alerts-snippets"))
            GM_gAlerts.show(account);
          
          // Play sound
          if (account.prefs.getBoolValue("notifications-sounds"))
            GM_gSound.preview(account.prefs.getValue("notifications-sounds-file"));
          
          // Switch account
          if (GM_getGlobal().getBoolValue("notifications-switch-account"))
            this._current = account.email;
        }
        case GM_NOTIFY_ACCOUNT_STATE_LOGGED_IN_NO_MAIL:
        {
          // Set check interval
          account.setTimer(account.prefs.getIntValue("notifications-check-interval"));
          break;
        }
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_PASSWORD:
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_CAPTCHA:
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_NETWORK:
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_SERVER:
        case GM_NOTIFY_ACCOUNT_STATE_ERROR_TIMEOUT:
          break;
        case GM_NOTIFY_ACCOUNT_ALERT_LINK_CLICK:
          // Loads the page
          GM_loadGmailPage("inbox", account, null, null);
          break;
      }
      
      // Select current account
      if (this._current == aTopic)
        this.selectAccount(aTopic, false);
    }
  }
}