// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

var GM_gPrefs = {
  _firstTime: true,
  _observer: null,
  _accounts: null,
  _current: null,
  
  load: function()
  {
    // Load observer
    this._observer = GM_CC["@mozilla.org/observer-service;1"].getService(GM_CI.nsIObserverService);
    this._observer.addObserver(this, GM_NOTIFY_ACCOUNT_CHANGE, false);
    
    // Set current
    this._current = GM_getCurrent();
    
    // Populate accounts
    this._populateAccounts();
    
    // Set first time
    this._firstTime = false;
  },
  
  unload: function()
  {
    // Remove observer
    this._observer.removeObserver(this, GM_NOTIFY_ACCOUNT_CHANGE);
  },
  
  accountAction: function(aAction)
  {
    switch (aAction)
    {
      case "add":
        window.openDialog('chrome://gmanager/content/accounts.xul', 'accounts', 'chrome,modal', 'add');
        break;
      case "remove":
        window.openDialog('chrome://gmanager/content/accounts.xul', 'accounts', 'chrome,modal', 'remove', this._current);
        break;
    }
  },
  
  _populateAccounts: function()
  {
    var accounts = GM_getManager().getAccounts({});
    var isAccounts = (accounts.length > 1);
    var accountsList = GM_getEBI("gm-prefs-accounts");
    var defaultsList = GM_getEBI("gm-prefs-compose-mailto-default");
    
    // Create list of accounts
    this._accounts = new Array();
    
    // Initialize lists
    this._initializeMenuList(accountsList, GM_getBundleString("prefs-account-settings"), false, isAccounts);
    this._initializeMenuList(defaultsList, GM_getBundleString("prefs-specific-account"), true, isAccounts);
    
    // Append lists
    for (var i = 0; i < accounts.length; i++)
    {
      var email = "global";
      
      // Load preferences
      accounts[i].prefs.load();
      
      if (accounts[i].type != "global")
      {
        email = accounts[i].email;
        
        accountsList.appendChild(this._createMenuItem(email, false));
        defaultsList.appendChild(this._createMenuItem(email, true));
        
        if (this._current == email)
          accountsList.parentNode.selectedItem = accountsList.lastChild;
      }
      
      // Store account
      this._accounts[email] = accounts[i];
    }
    
    // Select account
    this.selectAccount(accountsList.parentNode.selectedIndex);
  },
  
  _initializeMenuList: function(aList, aLabel, aApply, aSeparator)
  {
    // Remove list items
    while (aList.hasChildNodes())
      aList.removeChild(aList.lastChild);
    
    aList.appendChild(this._createMenuItem(aLabel, aApply));
    
    if (aSeparator)
      aList.appendChild(document.createElement("menuseparator"));
    
    // Select first item as default
    aList.parentNode.selectedIndex = 0;
  },
  
  _createMenuItem: function(aLabel, aApply)
  {
    var item = document.createElement("menuitem");
    
    item.setAttribute("label", aLabel);
    item.setAttribute("apply", (aApply ? "true" : "false"));
    
    return item;
  },
  
  selectAccount: function(aIndex)
  {
    var isGlobal = (aIndex == 0);
    var list = GM_getEBI("gm-prefs-accounts");
    
    // Save preferences
    if (this._current != null && !this._firstTime)
      this._savePrefs(this._current);
    else
      this._loadPrefs("global");
    
    // Selects the item by index
    list.parentNode.selectedIndex = aIndex;
    
    if (!isGlobal)
    {
      // Specific account
      var email = list.parentNode.selectedItem.getAttribute("label");
      var account = this._accounts[email];
      
      GM_getEBI("gm-prefs-general-alias").value = account.alias;
      GM_getEBI("gm-prefs-general-password").value = account.password;
      GM_getEBI("gm-prefs-general-password-remember").checked = account.remember;
    }
    
    // Current account
    this._current = (isGlobal ? "global" : email);
    
    // Load preferences
    this._loadPrefs(this._current);
    
    // Update fields
    this._update();
    
    var tabs = GM_getEBI("gm-prefs-tabs").childNodes;
    
    for (var i = 0; i < tabs.length - 1; i++)
      tabs[i].collapsed = (tabs[i].getAttribute("type") == "global" ? !isGlobal : isGlobal);
    
    if (GM_getEBI("gm-prefs-tabs").selectedItem.collapsed)
      GM_getEBI("gm-prefs-tabs").selectedIndex = (isGlobal ? 0 : 4);
  },
  
  _loadPrefs: function(aEmail)
  {
    var prefs = this._accounts[aEmail].prefs.nodeList;
    
    for (var i = 0; i < prefs.length; i++)
    {
      var pref = prefs.item(i);
      var id = pref.getAttribute("id");
      var value = pref.getAttribute("value");
      
      switch (pref.getAttribute("type"))
      {
        case "Bool":
          GM_getEBI("gm-prefs-" + id).checked = (value == "true" ? true : false);
          break;
        case "List":
          GM_getEBI("gm-prefs-" + id).parentNode.selectedIndex = parseInt(value);
          break;
        case "Int":
        case "Char":
          GM_getEBI("gm-prefs-" + id).value = value;
          break;
        default:
          break;
      }
    }
  },
  
  _savePrefs: function(aEmail)
  {
    var account = this._accounts[aEmail];
    var prefs = account.prefs.nodeList;
    
    if (aEmail != "global")
    {
      account.alias = GM_getEBI("gm-prefs-general-alias").value
      account.password = GM_getEBI("gm-prefs-general-password").value
      account.remember = GM_getEBI("gm-prefs-general-password-remember").checked;
    }
    
    for (var i = 0; i < prefs.length; i++)
    {
      var pref = prefs.item(i);
      var id = pref.getAttribute("id");
      
      switch (pref.getAttribute("type"))
      {
        case "Bool":
          pref.setAttribute("value", GM_getEBI("gm-prefs-" + id).checked)
          break;
        case "List":
          pref.setAttribute("value", GM_getEBI("gm-prefs-" + id).parentNode.selectedIndex);
          break;
        case "Int":
        case "Char":
          pref.setAttribute("value", GM_getEBI("gm-prefs-" + id).value);
          break;
        default:
          break;
      }
    }
  },
  
  importPrefs: function()
  {
    var file = GM_getManager().importFile(window);
    
    if (file == null)
      alert(GM_getBundleString("prefs-import-error"));
    else
    {
      if (file == "")
        return;
      
      // Populate accounts
      this._populateAccounts();
      
      // Notify of account prefs change
      this._observer.notifyObservers(null, GM_NOTIFY_ACCOUNT_PREFS_CHANGE, null);
      
      alert(GM_getBundleString("prefs-import-success"));
    }
  },
  
  exportPrefs: function()
  {
    if (GM_getManager().exportFile(window) != null)
      alert(GM_getBundleString("prefs-export-success"));
  },
  
  sound: function(aAction)
  {
    switch (aAction)
    {
      case "select":
      {
        var file = GM_gSound.select();
        
        if (file != null)
        {
          // Sets the textbox with file path
          GM_getEBI("gm-prefs-notifications-sounds-file").value = file;
          
          // Update fields
          this._update();
        }
        break;
      }
      case "preview":
      {
        var file = GM_getEBI("gm-prefs-notifications-sounds-file").value;
        GM_gSound.preview(file);
        break;
      }
    }
  },
  
  action: function(aEvent)
  {
    var localName = aEvent.originalTarget.localName;
    var isApply = aEvent.originalTarget.getAttribute("apply");
    
    if (localName == "button" || localName == "tab" || isApply == "false")
      return;
    
    // Update fields
    this._update();
    
    // Enable apply button
    GM_getEBI("gm-prefs-apply").disabled = false;
  },
  
  _update: function()
  {
    var isDisabled = null;
    
    // Remove button
    isDisabled = (this._current != "global");
    GM_getEBI("gm-prefs-accounts-remove").disabled = !isDisabled;
    
    // Mailto: links
    isDisabled = GM_getEBI("gm-prefs-compose-mailto-links").checked;
    GM_getEBI("gm-prefs-compose-mailto-default-label").disabled = !isDisabled;
    GM_getEBI("gm-prefs-compose-mailto-default").parentNode.disabled = !isDisabled;
    
    // Context menu
    isDisabled = GM_getEBI("gm-prefs-compose-context-menu").checked;
    GM_getEBI("gm-prefs-compose-context-menu-mailto").disabled = !isDisabled;
    GM_getEBI("gm-prefs-compose-context-menu-position-label").disabled = !isDisabled;
    GM_getEBI("gm-prefs-compose-context-menu-position").disabled = !isDisabled;
    GM_getEBI("gm-prefs-compose-context-menu-position-range").hidden = !isDisabled;
    
    // Statusbar
    isDisabled = GM_getEBI("gm-prefs-toolbar-statusbar-display").checked;
    GM_getEBI("gm-prefs-toolbar-statusbar-always-last").parentNode.disabled = !isDisabled;
    
    // Statusbar position
    isDisabled = isDisabled && !GM_getEBI("gm-prefs-toolbar-statusbar-always-last").selected;
    GM_getEBI("gm-prefs-toolbar-statusbar-position").disabled = !isDisabled;
    GM_getEBI("gm-prefs-statusbar-position-range").hidden = !isDisabled;
    
    // Password remember
    isDisabled = (GM_getEBI("gm-prefs-general-password").value == "" || GM_getEBI("gm-prefs-general-never-save-passwords").checked);
    GM_getEBI("gm-prefs-general-password-remember").disabled = isDisabled;
    
    // Sounds
    isDisabled = GM_getEBI("gm-prefs-notifications-sounds").checked;
    GM_getEBI("gm-prefs-notifications-sounds-file").disabled = !isDisabled;
    GM_getEBI("gm-prefs-notifications-sound-browse").disabled = !isDisabled;
    
    // Sounds preview
    isDisabled = isDisabled && (GM_getEBI("gm-prefs-notifications-sounds-file").value != "");
    GM_getEBI("gm-prefs-notifications-sound-preview").disabled = !isDisabled;
    
    // Set context menu range
    GM_getEBI("gm-prefs-compose-context-menu-position-range").
      value = "0 - " + GM_getNumContextMenu();
    
    // Set statusbar range
    GM_getEBI("gm-prefs-statusbar-position-range").
      value = "0 - " + GM_getNumStatusbar();
  },
  
  apply: function(aClose)
  {
    // Save preferences
    if (this._current != null)
      this._savePrefs(this._current);
    
    if (GM_getEBI("gm-prefs-general-never-save-passwords").checked)
    {
      for (var email in this._accounts)
        if (email != "global")
          this._accounts[email].remember = false;
    }
    
    // Save to file
    GM_getManager().save();
    
    // Disable apply button
    GM_getEBI("gm-prefs-apply").disabled = true;
    
    // Notify of account prefs change
    this._observer.notifyObservers(null, GM_NOTIFY_ACCOUNT_PREFS_CHANGE, null);
    
    if (aClose)
      this.cancel();
  },
  
  cancel: function()
  {
    self.close();
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic == GM_NOTIFY_ACCOUNT_CHANGE)
    {
      if (aData == "pass" || aData == "migrate")
        return;
      
      // Reset current
      this._current = null;
      
      // Populate accounts
      this._populateAccounts();
      
      switch (aData)
      {
        case "add":
          var index = GM_getEBI("gm-prefs-accounts").childNodes.length - 1;
          this.selectAccount(index);
          break;
      }
    }
  }
}